Index: src/uk/me/parabola/imgfmt/app/Coord.java
===================================================================
--- src/uk/me/parabola/imgfmt/app/Coord.java	(revision 2724)
+++ src/uk/me/parabola/imgfmt/app/Coord.java	(working copy)
@@ -30,8 +30,6 @@
  * You can create one of these with lat/long by calling the constructor with
  * double args.
  *
- * This is an immutable class.
- *
  * @author Steve Ratcliffe
  */
 public class Coord implements Comparable<Coord> {
@@ -314,4 +312,8 @@
 		return toOSMURL(17);
 	}
 
+	public void resetHighwayCount() {
+		highwayCount = 0;
+	}
+
 }
Index: src/uk/me/parabola/mkgmap/osmstyle/StyledConverter.java
===================================================================
--- src/uk/me/parabola/mkgmap/osmstyle/StyledConverter.java	(revision 2724)
+++ src/uk/me/parabola/mkgmap/osmstyle/StyledConverter.java	(working copy)
@@ -347,6 +347,8 @@
 		findUnconnectedRoads();
 		filterCoordPOI();
 		removeShortArcsByMergingNodes();
+		resetHighwayCounts();
+		setHighwayCounts();
 		// make sure that copies of modified roads are have equal points 
 		for (int i = 0; i < lines.size(); i++){
 			Way line = lines.get(i);
@@ -1745,6 +1747,9 @@
 		log.info("Maintaining highway counters");
 		long lastId = 0;
 		for (Way way :roads){
+			if (way == null)
+				continue;
+			
 			if (way.getId() == lastId) {
 				log.debug("Road with identical id:", way.getId());
 				continue;
@@ -1756,6 +1761,28 @@
 			}
 		}
 	}
+	
+	/**
+	 * Increment the highway counter for each coord of each road.
+	 * As a result, all road junctions have a count > 1. 
+	 */
+	private void resetHighwayCounts(){
+		log.info("Resetting highway counters");
+		long lastId = 0;
+		for (Way way :roads){
+			if (way == null)
+				continue;
+			
+			if (way.getId() == lastId) {
+				continue;
+			}
+			lastId = way.getId();
+			List<Coord> points = way.getPoints();
+			for (Coord p:points){
+				p.resetHighwayCount();
+			}
+		}
+	}
 
 	/**
 	 * Detect roads that do not share any node with another road.
